function [Al,par,phi,PI,we,lk,aic,bic,La,Pio,seAl,separ,sephi,setau,J] = est_marg_long_lm(Y,X,Y0,lev,type,k,tol,Al,par,phi,PI)

%        [Al,par,phi,PI,we,lk,aic,bic,La,Pio,seAl,separ,sephi,setau,J] = est_marg_long_lm(Y,X,Y0,lev,type,k,tol,Al,par,phi,PI)
%
% Fit the model described in the paper "A multivariate extension of the dynamic logit model for longitudinal 
% data based on a latent Markov heterogeneity structure"
%
% INPUT:
% Y:     matrix (nt x n x T) of response variables with all zeros and 1 for
%        the observed resonse configuration (nt = number of response configurations)
% X:     matrix (np+ni x npar x n x T) of covariates with eventually includes lagged
%        response (np = number of marginal logits, ni = number of log-odds ratios, n
%        par = number of parameters)
% Y0:    matrix (n x ny0) vector with rows used to model the initial
%        probabilities of the latent process
% lev:   vector containing the number of levels of each variable
% type:  vector with elements 'l', 'g', 'c', 'r' indicating the type of logit
% k:     number of latent states
% tol:   tolerance for the convergence (optional)
% Xi:    starting value for matrix of intercepts for each latent state (optional)
% par:   starting value for vector of regression parameters (optional)
% de:    starting value for parameter vector for initial probabilities (optional)
% PI:    starting value for transition matrix (optional)
%
% OUTPUT:
% Al:    matrix of intercepts for each latent state
% par:   estimate of the vector of regression parameters
% phi:   parameter vector for initial probabilities
% PI:    transition matrix
% we:    posterior weight for each latent state
% lk:    maximum log-likelihood
% aic:   AIC index
% bic:   BIC index
% La:    matrix of initial probabilities
% Pio:   array of probabilities of response configurations
% sephi: standard error for de
% sepsi: standard error for psi (logits for PI)
% seXi:  standard error for Al
% sebe:  standard error for be
% sephi: standard error for phi
% setau: standard error for tau
% J:     obsserved information matrix

% preliminaries
  l0 = size(Y0,2);
  if nargin<7, tol=[]; end
  if isempty(tol), tol = 10^-4; end
  r = length(lev);
  [C,M,G] = marg_param(lev,type); 
  Gt = G';
  G1 = [zeros(1,k-1); eye(k-1)];
  [ne,npar,n,T] = size(X);
  nt = prod(lev); 
  np = sum(lev)-r;
  ni = 0;
  for h = 1:r-1,
    for j = h+1:r,
      ni = ni+(lev(h)-1)*(lev(j)-1);
    end
  end
% starting values
  W = zeros(k-1,(k-1)*(1+l0),n); I2 = eye(k-1);
  for i = 1:n,
    W(:,:,i) = kron(I2,[1 Y0(i,:)]);
  end
  if nargin<8,
    Pim = sum(Y,3)+0.05*T; Eta = C*log(M*Pim);
    Eta = kron(Eta,ones(1,T));
    eta = reshape(Eta,ne*n*T,1);
    Z = reshape(permute(X,[1 4 3 2]),n*ne*T,size(X,2));
    Z = [kron(ones(n*T,1),[eye(np); zeros(ne-np,np)]) Z];
    par = inv(Z'*Z)*Z'*eta;
    eta = par(1:np); par(1:np) = [];
    if k==1,
      Al = eta(1:np);
    else
      Al = zeros(np,k); grid = -3:4/(k-1):3;
      for c = 1:k,
        Al(:,c) = eta(1:np)+grid(c);
      end
    end
    if k>1,
      phi = zeros((k-1)*(l0+1),1);
    end
    PI = ones(k)+9*eye(k); PI = diag(1./sum(PI,2))*PI;
  end
  al = reshape(Al,k*np,1);    
  par = [al; par];
  if k>1,
    La1 = zeros(n,k);
    for i = 1:n,
      La1(i,:) = (G1*W(:,:,i)*phi)';
    end
    La1 = exp(La1); La1 = diagv(1./sum(La1,2),La1);
  else
    La1 = ones(n,1);
  end
% compute initial log-likelihood
  Pi = zeros(nt,n,T,k); 
  I = [eye(np); zeros(ne-np,np)];
  RR = zeros(np+ni,np+ni,n,T,k);
  nnit = 0;
  for c = 1:k,
    u = zeros(1,k); u(c) = 1;
    D = kron(u,I);
    for t = 1:T,
      Eta = zeros(ne,n);
      Xt = X(:,:,:,t);
      for i = 1:n,
        Xi = [D Xt(:,:,i)];
        Eta(:,i) = Xi*par;
      end
      [Pi(:,:,t,c),er,nit,RR(:,:,:,t,c)] = glinvev(Eta,C,M,G);
      nnit = nnit+nit;
      if any(er), fprintf('err'); end
    end
  end
  nnit = nnit/(k*T);
  Pio = zeros(n,k,T);
  for c = 1:k,
    for t = 1:T,
      Pio(:,c,t) = sum(Y(:,:,t).*Pi(:,:,t,c),1)';
    end
  end
% apply recursion
  Q = rec1(Pio,La1,PI);
  pim = sum(Q(:,:,T),2);
  lk = sum(log(pim));
  disp('    n. states         step      log-lik.    diff.-log-lik.   diff.-par.');
  disp([k 0 lk]);
% iterate until convergence
  it = 0; it1 = 0; lko = lk; paro = 0;
  while (abs(lk-lko)>tol || it==0 || max(abs(par-paro))>10^-3) && it<50000,
    it = it+1; lko = lk;
% E-step
    [U,V] = rec2(Q,PI,Pio,pim);
    we = mean(mean(U,3),2);
% M-step
% update phi and PI
    if k>1
      mdphi = 1; it1 = 0;
      while (mdphi>tol && it1<1) || it1==0,
        it1 = it1+1;
        s = 0; F = 0;
        for i = 1:n,
          WG = W(:,:,i)'*G1'; la1 = La1(i,:)';
          s = s+WG*(U(:,i,1)-la1);
          F = F+WG*(diag(la1)-la1*la1')*WG';
        end
        dphi = pinv(F)*s; mdphi = max(abs(dphi));
        if mdphi>0.5,
          dphi = dphi/mdphi*0.5;
        end
        phi = phi+dphi;
        for i = 1:n,
          La1(i,:) = (G1*W(:,:,i)*phi)';
        end
        La1 = exp(La1); La1 = diagv(1./sum(La1,2),La1);
      end
    end
    PI = diag(1./sum(V,2))*V;
% update par
    it2 = 0; 
    U1 = permute(U,[2 1 3]);
    lkc = sum(sum(sum(log(Pio).*U1))); lkco = lk;
    while (abs(lkc-lkco)>tol && it2<1) || it2==0,
      it2 = it2+1;
      s = 0; F = 0;
      for c = 1:k,
        u = zeros(1,k); u(c) = 1;
        D = kron(u,I);
        for t = 1:T,
          pc = U(c,:,t)';
          Yt = Y(:,:,t); Pit = Pi(:,:,t,c); Xt = X(:,:,:,t);
          YGP = (Gt*(Yt-Pit));
          RRtc = RR(:,:,:,t,c);
          for i = 1:n,
            pit = Pit(:,i);
            Ri = RRtc(:,:,i);
            XRi = [D Xt(:,:,i)]'*Ri';
            s = s+pc(i)*XRi*YGP(:,i);
            XRi = XRi*Gt;
            F = F+pc(i)*XRi*(diag(pit)-pit*pit')*XRi';
          end
        end
      end
% update parameter vector
%      if any(any(isnan(F))), keyboard; end
      dpar = pinv(F)*s;
      mdpar = max(abs(dpar));
      if mdpar>0.5,
        dpar = dpar/mdpar*0.5;
      end
      paro = par;  par = par+dpar;
% compute nel log-likelihood
% invert parametrization
      flag = 1; nf = 0; Pi1 = Pi; RR1 = RR;
      while flag,
        flag = 0; nnit = 0;
        for c = 1:k,
          u = zeros(1,k); u(c) = 1;
          D = kron(u,I);
          for t = 1:T,
            Xt = X(:,:,:,t); Pit = Pi(:,:,t,c);
            for i = 1:n,
              Xi = [D Xt(:,:,i)];
              Eta(:,i) = Xi*par;
            end
            [Pi(:,:,t,c),er,nit,RR(:,:,:,t,c)] =  glinvev(Eta,C,M,G,Pit,RR(:,:,:,t,c));
            nnit = nnit+nit;
            if any(er), 
              flag = 1;
            end
          end
        end
        nnit = nnit/(k*T);
        if flag,
          nf = nf+1;
          fprintf('r');
          Pi = Pi1; RR = RR1;
          dpar = dpar/5;
          par = paro+dpar;
        end
      end
      for c = 1:k,
        for t = 1:T,
          Pio(:,c,t) = sum(Y(:,:,t).*Pi(:,:,t,c),1)';
        end
      end
      lkco = lkc; lkc = sum(sum(sum(log(Pio).*U1)));
    end    
% apply recursion
    Q = rec1(Pio,La1,PI);
    pim = sum(Q(:,:,T),2);
    lk = sum(log(pim));
% display results
    disp([k it lk lk-lko max(abs(par-paro))]);
  end
% compute standard errors
  if nargout>10,
    if k>1,
      PI1 = max(PI,eps);
      tau = [];
      for c = 1:k,
        ind = setdiff(1:k,c);
        tau = [tau; log(PI1(c,ind)'/PI1(c,c))];
      end
      par1 = [phi; tau; par];
    else
      par1 = par;
    end
    [lk1,s1] = lk_obs(par1,Y,X,W,lev,type,k,Pi,l0);
    J = -nder2('lk_obs',par1,Y,X,W,lev,type,k,Pi,l0);
    J = (J+J')/2;
    try
      se = sqrt(diag(pinv(J)));
      sephi = se(1:(k-1)*(l0+1)); se(1:(k-1)*(l0+1)) = [];
      setau = se(1:k*(k-1)); se(1:k*(k-1)) = [];
      seAl = reshape(se(1:k*np),np,k);
      separ = se(np*k+1:end);
    catch
      sephi = []; setau = []; seAl = []; separ = [];
    end
  end
% separate parameters and compute aic and bic
  La = La1;
  Al = reshape(par(1:k*np),np,k);
  par = par(np*k+1:end);
  if k==1, phi = []; end
  npa = k*np+npar+(k-1)*(l0+1)+k*(k-1);
  aic = -2*lk+2*(npa);
  bic = -2*lk+log(n)*(npa);
  disp(' ');
%--------------------------------------------------------------------------  
function Q = rec1(Pio,La,PI)

  [n,k,T] = size(Pio);  
  Q = zeros(n,k,T);
  Q(:,:,1) = Pio(:,:,1).*La;
  for t = 2:T,
    Q(:,:,t) = (Q(:,:,t-1)*PI).*Pio(:,:,t);
  end
%--------------------------------------------------------------------------    
function [U,V] = rec2(Q,PI,Pio,pim)

% preliminaries
  [n,k,T] = size(Pio);  
  U = zeros(k,n,T); V = 0;
% for t=T
  Q1 = Q(:,:,T-1); P1 = Pio(:,:,T);
  V1 = ones(k,n);
  for i = 1:n,
    Vi = (Q1(i,:)'*P1(i,:)).*PI/pim(i); V = V+Vi;
    U(:,i,T) = sum(Vi)';
  end
% for other t
  for t = T-1:-1:2,
    Q1 = Q(:,:,t-1); P1 = Pio(:,:,t); P2 = Pio(:,:,t+1);
    V1 = PI*(P2'.*V1); P3 = P1.*V1';
    for i = 1:n,
      Q1i = Q1(i,:)/pim(i);
      Vi = (Q1i'*P3(i,:)).*PI; V = V+Vi;
      U(:,i,t) = sum(Vi)';
      if t==2,
        U(:,i,1) = sum(Vi,2);
      end
    end
  end
%--------------------------------------------------------------------------
function [lk,s] = lk_obs(par,Y,X,W,lev,type,k,Pi,l0)

%        [lk,s] = lk_obs(par,Y,X,W,lev,type,k,Pi,l0)
%
% INPUT:
% Y:    matrix of response variables
% D:    design matrix for the covariates
% X:    matrix of covariates
% lev:  vector containing the number of levels of each variable
% type: vector with elements 'l', 'g', 'c', 'r' indicating the type of logit
% k:    number of latent classes
% par:  vector of regression parameters
% La:   starting value for initial probability vector
% PI:   starting value for transition matrix
%
% OUTPUT:
% lk:   log-likelihood
% s:    score of the observed log-likelihood

% preliminaries
  de = par(1:(k-1)*(l0+1));
  par(1:(k-1)*(l0+1)) = [];
  psi = par(1:k*(k-1));
  par(1:k*(k-1)) = [];
  r = length(lev); nt = prod(lev);
  [C,M,G] = marg_param(lev,type); 
  Gt = G'; G1 = [zeros(1,k-1); eye(k-1)];
  [ne,npar,n,T] = size(X);
  np = sum(lev)-r;
% compute initial log-likelihood
  I = [eye(np); zeros(ne-np,np)];
  Eta = zeros(ne,n);
  for c = 1:k,
    u = zeros(1,k); u(c) = 1;
    D = kron(u,I);
    dp = D*par(1:np*k);
    for t = 1:T,
      Xt = X(:,:,:,t);
      for i = 1:n,
        Eta(:,i) = dp+Xt(:,:,i)*par(np*k+1:end);
      end
      Pi(:,:,t,c) = glinvev(Eta,C,M,G,Pi(:,:,t,c));
    end
  end
  Pio = zeros(n,k,T);
  for c = 1:k,
    for t = 1:T,
      Pio(:,c,t) = sum(Y(:,:,t).*Pi(:,:,t,c),1)';
    end
  end
  if k>1,
    La = zeros(n,k);
    for i = 1:n,
      La(i,:) = (G1*W(:,:,i)*de)';
    end
    La = exp(La); La = diagv(1./sum(La,2),La);
    PI = zeros(k);
    for c = 1:k,
      G2 = eye(k); G2(c,:) = [];
      ind = (k-1)*(c-1)+1:c*(k-1);
      pii = exp(G2'*psi(ind)); pii = pii/sum(pii);
      PI(c,:) = pii;
    end
  else
    La = ones(n,1); PI = 1;
  end  
% apply recursion
  Q = rec1(Pio,La,PI); 
  pim = sum(Q(:,:,T),2);
  lk = sum(log(pim));
% compute derivative  
  if nargout>1,
% with respect to de    
    if k>1,
      Qn = rec3(Pio,PI);
      s1 = 0;
      for i = 1:n,
        lai = La(i,:)';
        s1 = s1+W(:,:,i)'*G1'*(diag(lai)-lai*lai')*Qn(i,:)'/pim(i);
      end
      Qn = rec4(Pio,La,PI);
      s2 = [];
      for c = 1:k,
        G2 = eye(k); G2(c,:) = [];
        pii = PI(c,:)';
        s2 = [s2; G2*(diag(pii)-pii*pii')*Qn(c,:)'];
      end
    else
      s1 = []; s2 = [];
    end
% E-step
    U = rec2(Q,PI,Pio,pim);
    s = 0; S = zeros(np*k+npar,n);
    for c = 1:k,
      u = zeros(1,k); u(c) = 1;
      D = kron(u,I);
      for t = 1:T,
        Pit = Pi(:,:,t,c); Xt = X(:,:,:,t);
        DP = 1./(M*Pit); YGP = (Gt*(Y(:,:,t)-Pit))';
        kron(Pit,ones(1,nt));
        reshape(DP,size(M,1)*n,1);
        for i = 1:n,
          Ri = inv(C*((DP(:,i)*Pit(:,i)').*M)*G);
          ve = YGP(i,:)*Ri;
          S(:,i) = [ve*D ve*Xt(:,:,i)];
        end
        s = s+S*U(c,:,t)';
      end
    end
% complete vector
    s = [s1; s2; s];
  end
%--------------------------------------------------------------------------
function D = nder2(fname,x,varargin)

%        D = nder2(fname,x,varargin)
%
% computes numerically the derivatives of y' with respect to x
% where y = fname(x)

tol = 10^-6;
x = x-tol/2;
[a,d0] = feval(fname,x,varargin{:});
nx = length(x); ny = length(d0);
D = zeros(nx,ny);
for i = 1:nx,
  o = zeros(nx,1); o(i) = tol;
  [a,d1] = feval(fname,x+o,varargin{:}); d = (d1-d0)/tol;
  D(i,:) = d';
%  disp(D(1:i,1:i));
  fprintf('.');
end
%--------------------------------------------------------------------------
function Q = rec3(Pio,PI)

  [n,k,T] = size(Pio);
  Q = zeros(n,k,k);
  for c = 1:k,
    Q(:,c,c) = Pio(:,c,1);
  end
  for c = 1:k,
    for t = 2:T,
      Q(:,:,c) = (Q(:,:,c)*PI).*Pio(:,:,t);
    end
  end
  Q = permute(sum(Q,2),[1 3 2]);
%--------------------------------------------------------------------------
function N = diagv(v,M)

%        N = diagv(v,M)
%
% Computes N = diag(v)*M avoiding the diag operator

N = (v*ones(1,size(M,2))).*M;
%--------------------------------------------------------------------------
function Qn = rec4(Pio,La,PI)

  [n,k,T] = size(Pio);  
  Q = Pio(:,:,1).*La; Qd = zeros(n,k,k,k);
  for t = 2:T,
    for c1 = 1:k,
      for c2 = 1:k,
        I = zeros(k); I(c1,c2) = 1;
        Qd(:,:,c1,c2) = (Q*I).*Pio(:,:,t)+(Qd(:,:,c1,c2)*PI).*Pio(:,:,t); 
      end
    end
    Q = (Q*PI).*Pio(:,:,t);
  end
  pim = sum(Q,2);
  Qd = permute(sum(Qd,2),[3 4 1 2]);
  Qn = zeros(k);
  for i = 1:n,
    Qn = Qn+Qd(:,:,i)/pim(i);
  end  